@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -16,11 +16,19 @@ import numpy as np
 
 from clasp import click
 import clasp.click_ext as clk
+import clasp.script_tools as cst
+
 import raytraverse
-from raytraverse.integrator import SunSkyIntegrator, MetricSet, Integrator, BaseIntegrator
-from raytraverse.sampler import SCBinSampler, SunSampler, SkySampler
-from raytraverse.scene import Scene, SunSetter, SunSetterLoc, SunSetterPositions
-from raytraverse.lightfield import SCBinField, SunField, SunViewField, StaticField
+# this is so readthedocs can build without installing as these modules depend
+# on c++ extensions that are not present.
+try:
+    from raytraverse.integrator import SunSkyIntegrator, Integrator, BaseIntegrator
+    from raytraverse.sampler import SCBinSampler, SunSampler, SkySampler
+    from raytraverse.scene import Scene, SunSetter, SunSetterLoc, SunSetterPositions
+    from raytraverse.lightfield import SCBinField, SunField, SunViewField, StaticField
+except ModuleNotFoundError as ex:
+    print(ex, file=sys.stderr)
+    pass
 
 
 @clk.pretty_name("NPY, TSV, FLOATS,FLOATS")
@@ -30,6 +38,8 @@ def np_load(ctx, param, s):
     trys np.load (numpy binary), then np.loadtxt (space seperated txt file)
     then split row by spaces and columns by commas.
     """
+    if s is None:
+        return s
     if os.path.exists(s):
         try:
             ar = np.load(s)
@@ -96,8 +106,8 @@ def main(ctx, out, config, n=None,  **kwargs):
               'analysis area')
 @click.option('-skyres', default=10.0,
               help='sky is subdivided accoring to a shirley-chiu disk to square'
-                   ' mapping, total number of sky bins will equal skyres^2.'
-                   ' solid angle of each patch will be 2*pi/(skyres^2)')
+                   ' mapping, approximate square patch size in degrees. set by:'
+                   ' int(np.floor(90/s)*2) to ensure an even number')
 @click.option('-ptres', default=2.0,
               help='resolution of point subdivision on analysis plane. units'
                    ' match radiance scene file')
@@ -133,6 +143,7 @@ def scene(ctx, **kwargs):
     analysis plane subdivision"""
     s = Scene(ctx.obj['out'], **kwargs)
     ctx.obj['scene'] = s
+    ctx.obj['initlf'] = []
     if kwargs['points']:
         for pt in s.pts():
             print('{}\t{}\t{}'.format(*pt))
@@ -140,7 +151,7 @@ def scene(ctx, **kwargs):
         print(f'\nScene {s.outdir}:', file=sys.stderr)
         print('='*60 + '\n', file=sys.stderr)
         print('Scene Geometry:', file=sys.stderr)
-        os.system(f'getinfo {s.scene}')
+        print(cst.pipeline([f'getinfo {s.scene}']), file=sys.stderr)
         print('Analysis Area:', file=sys.stderr)
         print('-'*60, file=sys.stderr)
         print(f'extents:\n{s.area.bbox}', file=sys.stderr)
@@ -181,7 +192,18 @@ run_opts = [
  click.option('--overwrite/--no-overwrite', default=False,
               help='execute run even if simulations exist'),
  click.option('--rebuild/--no-rebuild', default=False,
-              help='force rebuild kdtree')
+              help='force rebuild kdtree'),
+ click.option('--showsample/--no-showsample', default=True,
+                 help='show samples on dviews'),
+ click.option('--showweight/--no-showweight', default=True,
+                 help='show weights on dviews'),
+ click.option('-dpts', default=None, callback=np_load,
+              help="if given points to evaluate with --plotdview, this can be "
+                   "a .npy file, a whitespace "
+                   "seperated text file or entered as a string with commas "
+                   "between components of a point and spaces between points. "
+                   "in all cases each point requires 6 numbers x,y,z,dx,dy,dz "
+                   "so the shape of the array will be (N, 6)")
     ]
 
 
@@ -195,10 +217,15 @@ run_opts = [
               help='rtrace options to pass to the rcontrib call'
                    ' see the man pages for rtrace, rcontrib, and '
                    ' rcontrib -defaults for more information')
+@click.option('-dviewpatch',
+              callback=clk.split_float,
+              help="to plot direct view for a single patch,"
+                   "give an x,y,z direction")
 @clk.shared_decs(run_opts)
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def sky(ctx, plotdview=False, run=True, rmraw=True, overwrite=False,
-        rebuild=False, **kwargs):
+        rebuild=False, showsample=True, showweight=True, dpts=None,
+        dviewpatch=None, **kwargs):
     """the sky command intitializes and runs a sky sampler and then readies
     the results for integration by building a SCBinField. sky should be invoked
     before calling suns, as the sky contributions are used to select the
@@ -217,8 +244,10 @@ def sky(ctx, plotdview=False, run=True, rmraw=True, overwrite=False,
         except IOError:
             pass
     sk = SCBinField(s, rebuild=run or rebuild, rmraw=rmraw)
+    ctx.obj['initlf'].append(sk)
     if plotdview:
-        sk.direct_view()
+        sk.direct_view(showsample=showsample, showweight=showweight, dpts=dpts,
+                       srcidx=dviewpatch, res=1024)
 
 
 @main.command()
@@ -266,6 +295,11 @@ def sky(ctx, plotdview=False, run=True, rmraw=True, overwrite=False,
               help="if False, regenerates sun positions, because positions may"
                    " be randomly selected this will make any sunrun results"
                    " obsolete")
+@click.option('--printsuns/--no-printsuns', default=False,
+              help="print sun positions to stdout")
+@click.option('--skyfilter/--no-skyfilter', default=True,
+              help="use sky simulation to threshold possible solar positions"
+                   " (with --usepositions)")
 @click.option('--usepositions/--no-usepositions', default=False,
               help='if True, sun positions will be chosen from the positions'
                    ' listed in wea. if more than one position is a candidate'
@@ -276,7 +310,7 @@ def sky(ctx, plotdview=False, run=True, rmraw=True, overwrite=False,
                    'be achieved.')
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def suns(ctx, loc=None, wea=None, usepositions=False, plotdview=False,
-         **kwargs):
+         skyfilter=True, **kwargs):
     """the suns command provides a number of options for creating sun positions
     used by sunrun see wea and usepositions options for details
 
@@ -297,7 +331,8 @@ def suns(ctx, loc=None, wea=None, usepositions=False, plotdview=False,
         if wea is None:
             raise ValueError('option -wea is required when use positions is '
                              'True')
-        s = SunSetterPositions(ctx.obj['scene'], wea, **kwargs)
+        s = SunSetterPositions(ctx.obj['scene'], wea, skyfilter=skyfilter,
+                               **kwargs)
     elif loc is not None:
         s = SunSetterLoc(ctx.obj['scene'], loc, **kwargs)
     elif wea is not None:
@@ -308,6 +343,10 @@ def suns(ctx, loc=None, wea=None, usepositions=False, plotdview=False,
     if plotdview:
         s.direct_view()
     ctx.obj['suns'] = s
+    if kwargs['printsuns']:
+        for pt in s.suns:
+            print('{}\t{}\t{}'.format(*pt))
+
 
 
 @main.command()
@@ -341,7 +380,8 @@ def suns(ctx, loc=None, wea=None, usepositions=False, plotdview=False,
 @clk.shared_decs(run_opts)
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
 def sunrun(ctx, plotdview=False, run=True, rmraw=False, overwrite=False,
-           rebuild=False, **kwargs):
+           rebuild=False, showsample=True, showweight=True, dpts=None,
+           **kwargs):
     """the sunrun command intitializes and runs a sun sampler and then readies
     the results for integration by building a SunField."""
     if 'suns' not in ctx.obj:
@@ -360,6 +400,7 @@ def sunrun(ctx, plotdview=False, run=True, rmraw=False, overwrite=False,
         except FileNotFoundError as ex:
             print(f'Warning: {ex}', file=sys.stderr)
         else:
+            ctx.obj['initlf'].append(sv)
             if plotdview:
                 sv.direct_view()
     if kwargs['reflection']:
@@ -373,27 +414,25 @@ def sunrun(ctx, plotdview=False, run=True, rmraw=False, overwrite=False,
         except FileNotFoundError as ex:
             print(f'Warning: {ex}', file=sys.stderr)
         else:
+            ctx.obj['initlf'].append(su)
             if plotdview:
                 items = list(su.items())
                 if len(items) >= 20:
                     items = None
-                su.direct_view(items=items)
+                su.direct_view(items=items, showsample=showsample,
+                               showweight=showweight, dpts=dpts, res=1024)
 
 
 @main.command()
-@click.argument('skydef')
 @clk.shared_decs(run_opts)
+@click.option('-skydef', help='sky scene file (.rad)',
+              type=click.Path(exists=True, dir_okay=False))
 @click.option('-skyname',
               help='basename for result files')
 @click.option('-fdres', default=10,
               help='the final directional sampling resolution, yielding a'
                    ' grid of potential samples at 2^fdres x 2^fdres per'
                    ' hemisphere')
-@click.option('-accuracy', default=6.0,
-              help='final sampling level will set # of samples by the number of'
-                   ' samples with variance greater than 1/4 this number, which '
-                   'has units of radiance for bright sky conditions this should'
-                   ' be set to a correspondingly higher value (default is 6.0)')
 @click.option('-rcopts',
               default='-ab 6 -ad 3000 -as 1500 -st 0 -ss 16 -aa .1',
               help='rtrace options for sun reflection runs'
@@ -408,9 +447,13 @@ def sunrun(ctx, plotdview=False, run=True, rmraw=False, overwrite=False,
                    'successive call will load these ambient files, so care '
                    'must be taken to not change any parameters')
 @clk.shared_decs(clk.command_decs(raytraverse.__version__, wrap=True))
-def onesky(ctx, skydef, skyname=None, plotdview=False, run=True, rmraw=False,
-           overwrite=False, rebuild=False, **kwargs):
+def onesky(ctx, skydef=None, skyname=None, plotdview=False, run=True,
+           rmraw=False, overwrite=False, rebuild=False, showsample=True,
+           showweight=True, dpts=None, **kwargs):
     """the onesky command is for running one off sky definitions."""
+    if skydef is None:
+        print("Error: '-skydef' is required", file=sys.stderr)
+        raise click.Abort
     if 'scene' not in ctx.obj:
         clk.invoke_dependency(ctx, scene)
     scn = ctx.obj['scene']
@@ -428,15 +471,22 @@ def onesky(ctx, skydef, skyname=None, plotdview=False, run=True, rmraw=False,
     except FileNotFoundError as ex:
         print(f'Warning: {ex}', file=sys.stderr)
     else:
+        ctx.obj['initlf'].append(su)
         if plotdview:
             items = list(su.items())
             if len(items) >= 20:
                 items = None
-            su.direct_view(items=items)
+            su.direct_view(items=items, showsample=showsample,
+                           showweight=showweight, dpts=dpts, res=1024)
             if su.view is not None:
                 su.view.direct_view()
 
 
+allmetrics = ["illum", "avglum", "lum2", "ugr", "dgp", "tasklum",
+              "backlum", "dgp_t1", "dgp_t2", "threshold", "pwsl2",
+              "view_area", "density", "reldensity", "lumcenter"]
+
+
 @main.command()
 @click.option('-loc', callback=clk.split_float,
               help='specify the scene location (if not specified in -wea or to'
@@ -470,6 +520,9 @@ def onesky(ctx, skydef, skyname=None, plotdview=False, run=True, rmraw=False,
               help='number of nearby rays to use for interpolation of hdr'
                    'output (weighted by a gaussian filter). this does'
                    'not apply to metric calculations')
+@click.option('-blursun', default=1,
+              help='spread sun to mimic camera or human eye, 1 is no blur'
+                   ' a value of 4 will double the apparent radius')
 @click.option('-vname', default='view',
               help='name to include with hdr outputs')
 @click.option('-static',
@@ -494,7 +547,7 @@ def onesky(ctx, skydef, skyname=None, plotdview=False, run=True, rmraw=False,
 @click.option('-metricset', default="illum avglum lum2 dgp ugr",
               callback=clk.split_str,
               help='which metrics to return items must be in: '
-                   f'{", ".join(MetricSet.allmetrics)}')
+                   f'{", ".join(allmetrics)}')
 @click.option('-threshold', default=2000.0,
               help='Threshold factor; if factor is larger than 100, it is used '
                    'as constant threshold in cd/m2. If factor is less or equal '
@@ -521,7 +574,7 @@ def onesky(ctx, skydef, skyname=None, plotdview=False, run=True, rmraw=False,
 def integrate(ctx, loc=None, wea=None, skyro=0.0, ground_fac=0.15, pts=None,
               skyonly=False, sunonly=False, hdr=True, metric=True, res=800, interp=12,
               vname='view', static=None, header=False, metricset="",
-              tradius=30.0, **kwargs):
+              tradius=30.0, blursun=1, **kwargs):
     """the integrate command combines sky and sun results and evaluates the
     given set of positions and sky conditions"""
     if 'scene' not in ctx.obj:
@@ -539,7 +592,7 @@ def integrate(ctx, loc=None, wea=None, skyro=0.0, ground_fac=0.15, pts=None,
         if 'suns' not in ctx.obj:
             clk.invoke_dependency(ctx, suns)
         sns = ctx.obj['suns']
-        su = SunField(scn, sns)
+        su = SunField(scn, sns, blursun=blursun)
         if sunonly:
             itg = Integrator(su, wea=wea, loc=loc, skyro=skyro,
                              ground_fac=ground_fac)
@@ -587,6 +640,9 @@ def printconfig(ctx, returnvalue, **kwargs):
             print('Has sunview lightfield data:',
                   os.path.isfile(f'{s.outdir}/sunview_kd_data.pickle'),
                   file=sys.stderr)
+            for lf in ctx.obj['initlf']:
+                print(f"lightfield \"{lf.prefix}\" covers {lf.size['lfang']:.02f} "
+                      f"steradians with {lf.size['lfcnt']} rays", file=sys.stderr)
     except KeyError:
         pass
     try:
