@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,7 +9,7 @@
 """mapper objects"""
 
 __all__ = ['BaseIntegrator', 'Integrator', 'SunSkyIntegrator',
-           'MetricSet', 'PositionIndex']
+           'MetricSet', 'PositionIndex', 'retina']
 
 from raytraverse.integrator.baseintegrator import BaseIntegrator
 from raytraverse.integrator.integrator import Integrator
