@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -47,10 +47,10 @@ class Integrator(BaseIntegrator):
         ground reflectance
     """
 
-    rowheaders = dict(idx=["pt-idx", "sky-idx"],
-                      sensor=["x", "y", "z", "dx", "dy", "dz"],
-                      err=["pt-err", "sun-err"],
-                      sky=["sun-x", "sun-y", "sun-z", "dir-norm", "diff-horiz"])
+    rowheaders0 = dict(idx=["pt-idx", "sky-idx"],
+                       sensor=["x", "y", "z", "dx", "dy", "dz"])
+    rowheaders1 = dict(err=["pt-err", "sun-err"],
+                       sky=["sun-x", "sun-y", "sun-z", "dir-norm", "diff-horiz"])
 
     def __init__(self, lightfield, wea=None, loc=None, skyro=0.0,
                  ground_fac=0.15):
@@ -149,6 +149,7 @@ class Integrator(BaseIntegrator):
                 if loc is None:
                     loc = skycalc.get_loc_epw(dat)
                 skydat = skycalc.read_epw(dat)
+        skydat = np.atleast_2d(skydat)
         if skydat.shape[1] == 4:
             xyz = translate.aa2xyz(skydat[:, 0:2])
             skydat = np.hstack((xyz, skydat[:, 2:]))
