@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -30,11 +30,12 @@ class SunField(LightFieldKD):
         build kd-tree even if one exists
     """
 
-    def __init__(self, scene, suns, rebuild=False, rmraw=False):
+    def __init__(self, scene, suns, rebuild=False, rmraw=False, **kwargs):
         #: raytraverse.sunsetter.SunSetter
         self.suns = suns
         #: raytraverse.lightfield.SunViewField
-        self.view = SunViewField(scene, suns, rebuild=False, rmraw=rmraw)
+        self.view = SunViewField(scene, suns, rebuild=False, rmraw=rmraw,
+                                 **kwargs)
         super().__init__(scene, rebuild=rebuild, prefix='sun', rmraw=rmraw,
                          fvrays=scene.maxspec)
 
@@ -76,6 +77,9 @@ class SunField(LightFieldKD):
     def items(self):
         return self.d_kd.keys()
 
+    def ptitems(self, i):
+        return [j for j in self.items() if j[0] == i]
+
     def keymap(self):
         npts = self.scene.area.npts
         shape = (npts, self.suns.suns.shape[0] + 1)
@@ -103,9 +107,11 @@ class SunField(LightFieldKD):
             omega = np.concatenate((omega, [svw[2]]))
         return rays, omega, lum
 
-    def direct_view(self, res=512, showsample=True, items=None):
+    def direct_view(self, res=512, showsample=True, showweight=True,
+                    dpts=None, items=None, srcidx=None):
         """create a summary image of lightfield for each vpt"""
-        super().direct_view(res=res, showsample=showsample, items=items)
+        super().direct_view(res=res, showsample=showsample,
+                            showweight=showweight, dpts=dpts, items=items)
         if items is not None:
             return None
         for i in super().items():
