@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -61,6 +61,9 @@ class SunSkyPt(LightFieldKD):
     def items(self):
         return self._items
 
+    def ptitems(self, i):
+        return [j for j in self.items() if j[0] == i]
+
     def apply_coef(self, pi, coefs):
         coefs = np.asarray(coefs)
         if coefs.size == 1:
@@ -154,6 +157,7 @@ class SunSkyPt(LightFieldKD):
         vecs = {}
         omegas = {}
         lums = {}
+        self.scene.log(self, "Interpolating kd-trees")
         with ProcessPoolExecutor(io.get_nproc()) as exc:
             futures = []
             for sidx in self.items():
