@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -55,6 +55,7 @@ class Sampler(object):
                  bands=1, engine_args="", nproc=None, **kwargs):
         name = type(self).__name__
         self._staticscene = True
+        scene.log(self, "Initializing")
         if name == "Sampler":
             raise NotImplementedError("Sampler base class should not be "
                                       "instantiated directly")
@@ -93,6 +94,7 @@ class Sampler(object):
         self._vecfiles = []
 
     def __del__(self):
+        self.scene.log(self, "Closed")
         if not self._staticscene:
             try:
                 os.remove(self.compiledscene)
@@ -234,7 +236,7 @@ class Sampler(object):
         self._vecfiles.append((outif, outf))
         return outf
 
-    def _plot_p(self, p):
+    def _plot_p(self, p, suffix=".hdr"):
         ps = p.reshape(self.weights.shape)
         outshape = (1024, 512)
         res = outshape[-1]
@@ -247,9 +249,9 @@ class Sampler(object):
                 ij = translate.uv2ij(uv, w.shape[-1])
                 ptidx = np.ravel_multi_index((i, j), self.scene.area.ptshape)
                 outw = (f"{self.scene.outdir}_{self.stype}_weights_"
-                        f"{ptidx:04d}_{self.idx+1:02d}.hdr")
+                        f"{ptidx:04d}_{self.idx+1:02d}{suffix}")
                 outp = (f"{self.scene.outdir}_{self.stype}_detail_"
-                        f"{ptidx:04d}_{self.idx+1:02d}.hdr")
+                        f"{ptidx:04d}_{self.idx+1:02d}{suffix}")
                 img = w[ij[:, 0], ij[:, 1]].reshape(outshape)
                 io.array2hdr(np.where(mask, img, 0), outw)
                 img = ps[i, j][ij[:, 0], ij[:, 1]].reshape(outshape)
@@ -268,6 +270,16 @@ class Sampler(object):
                         f"{ptidx:04d}_{level:02d}.hdr")
                 io.array2hdr(img, outf)
 
+    def _linear(self, x, x1, x2):
+        if len(self.levels) <= 2:
+            return (x1, x2)[x]
+        else:
+            return (x2 - x1)/(len(self.levels) - 2)*(x - 1) + x1
+
+    def threshold(self, idx):
+        """threshold for determining sample count"""
+        return self.accuracy * self._linear(idx, 2**-8, .125)
+
     def draw(self):
         """draw samples based on detail calculated from weights
         detail is calculated across direction only as it is the most precise
@@ -288,11 +300,11 @@ class Sampler(object):
             p = draw.get_detail(self.weights, daxes)
             if self.plotp:
                 self._plot_p(p)
+            # a cooling parameter towards deterministic sampling at final level
+            bound = self._linear(self.idx, .5, 0)
             # draw on pdf
-            # threshold is set to accurracy at final
-            threshold = (self.accuracy *
-                         4**(self.idx - len(self.levels)))
-            pdraws = draw.from_pdf(p, threshold)
+            pdraws = draw.from_pdf(p, self.threshold(self.idx),
+                                   lb=1-bound, ub=1+bound)
         return pdraws
 
     def update_pdf(self, si, lum):
@@ -310,6 +322,13 @@ class Sampler(object):
 
     def run_callback(self):
         outf = f'{self.scene.outdir}/{self.stype}_vecs.out'
+        # output traced rays for performance comparison
+        # outft = f'{self.scene.outdir}/{self.stype}_vecs_trace.out'
+        # f = open(outft, 'wb')
+        # for idxf, vecf in self._vecfiles:
+        #     fsrc = open(vecf, 'rb').read()
+        #     f.write(fsrc)
+        # f.close()
         f = open(outf, 'wb')
         for idxf, vecf in self._vecfiles:
             fsrc = open(vecf, 'rb')
@@ -343,13 +362,12 @@ class Sampler(object):
     # @profile
     def run(self):
         """execute sampler"""
-
         allc = 0
         f = open(f'{self.scene.outdir}/{self.stype}_vals.out', 'wb')
         f.close()
-        print('Sampling...', file=sys.stderr)
+        self.scene.log(self, f"Started sampling {self.stype}")
         hdr = ['level', 'shape', 'samples', 'rate', 'filesize (MB)']
-        print('{:>8}  {:>25}  {:<10}  {:<8}  {}'.format(*hdr), file=sys.stderr)
+        self.scene.log(self, '\t'.join(hdr))
         fsize = 0
         for i in range(self.idx, self.levels.shape[0]):
             shape = np.concatenate((self.scene.area.ptshape, self.levels[i]))
@@ -359,25 +377,22 @@ class Sampler(object):
             if draws is None:
                 srate = 0.0
                 row = [f'{i + 1} of {self.levels.shape[0]}', str(shape),
-                       0, f"{srate:.02%}", fsize]
-                print('{:>8}  {:>25}  {:<10}  {:<8}  {:.03f}'.format(*row),
-                      file=sys.stderr)
+                       '0', f"{srate:.02%}", f'{fsize:.03f}']
+                self.scene.log(self, '\t'.join(row))
             else:
                 self.levelsamples[self.idx] = draws.size
                 si, vecs = self.sample_idx(draws)
                 srate = si.shape[1]/np.prod(shape)
                 fsize += 4*self.bands*self.srcn*si.shape[1]/1000000
                 row = [f'{i+1} of {self.levels.shape[0]}', str(shape),
-                       si.shape[1], f"{srate:.02%}", fsize]
-                print('{:>8}  {:>25}  {:<10}  {:<8}  {:.03f}'.format(*row),
-                      file=sys.stderr)
+                       str(si.shape[1]), f"{srate:.02%}", f'{fsize:.03f}']
+                self.scene.log(self, '\t'.join(row))
                 vecf = self.dump_vecs(vecs, si)
                 lum = self.sample(vecf)
                 self.update_pdf(si, lum)
                 a = lum.shape[0]
                 allc += a
-        print("-"*70, file=sys.stderr)
         srate = allc/self.weights.size
-        row = ['total sampling:', allc, f"{srate:.02%}", fsize]
-        print('{:<35}  {:<10}  {:<8}  {:.03f}'.format(*row), file=sys.stderr)
+        row = ['total sampling:', '-', str(allc), f"{srate:.02%}", f'{fsize:.03f}']
+        self.scene.log(self, '\t'.join(row))
         self.run_callback()
