@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Stephen Wasilewski
+# Copyright (c) 2020 Stephen Wasilewski, HSLU and EPFL
 # =======================================================================
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -32,13 +32,14 @@ class SunSetterPositions(SunSetter):
         if True reloads existing sun positions, else always generates new
     """
 
-    def __init__(self, scene, wea, skyro=0.0, **kwargs):
+    def __init__(self, scene, wea, skyro=0.0, skyfilter=True, **kwargs):
         #: raytraverse.scene.Scene
         self.scene = scene
         #: float: ccw rotation (in degrees) for sky
         self.skyro = skyro
         #: raytraverse.scene.SkyInfo
         self.candidates = wea
+        self.skyfilter = skyfilter
         super().__init__(scene, skyro=skyro, **kwargs)
 
     @property
@@ -55,7 +56,7 @@ class SunSetterPositions(SunSetter):
     def candidates(self, wea):
         """load candidate sun positions"""
         try:
-            dat = np.loadtxt(wea)
+            dat = np.atleast_2d(np.loadtxt(wea))
             if dat.shape[1] == 3:
                 dummydir = np.ones((dat.shape[0], 1))
                 self._candidates = np.hstack((translate.norm(dat), dummydir))
@@ -78,12 +79,13 @@ class SunSetterPositions(SunSetter):
         uvsize = self.sunres
         cbins = translate.uv2bin(translate.xyz2uv(self.candidates[:, 0:3],
                                                   normalize=True, flipu=False),
-                                 self.scene.skyres)
+                                 uvsize)
         cidxs = np.arange(cbins.size)
         sbins = np.arange(uvsize**2)
-        skyb = self.load_sky_facs()
-        if skyb is 1:
-            skyb = np.full(sbins.shape[0], 1)
+        if self.skyfilter:
+            skyb = self.load_sky_facs()
+        else:
+            skyb = np.ones(uvsize*uvsize)
         idxs = []
         for b in sbins:
             if skyb[b] > self.srct:
