@@ -2,12 +2,25 @@
 # -*- coding: utf-8 -*-
 
 """Tests for raytraverse.metric"""
+import os
+
+import pytest
+from raytraverse import io
 from raytraverse.mapper import ViewMapper
 from raytraverse.integrator import MetricSet, PositionIndex
 import numpy as np
 
 
-def test_get_pos_idx():
+@pytest.fixture(scope="module")
+def tmpdir(tmp_path_factory):
+    data = str(tmp_path_factory.mktemp("data"))
+    cpath = os.getcwd()
+    os.chdir(data)
+    yield data
+    os.chdir(cpath)
+
+
+def test_get_pos_idx(tmpdir):
     exp = [[2.425470197687602, 2.705124422836401, 3.250879646438889,
             4.510683974902716, 4.6, 8.968471163183748, 16.0, 16.0, 16.0, 16.0],
            [2.705124422836401, 2.696633390419641, 2.7728963523408092,
@@ -44,3 +57,28 @@ def test_get_pos_idx():
     posfinder = PositionIndex()
     posidx = posfinder.positions(vm, fimg).reshape(res, res)
     assert np.allclose(posidx, exp)
+
+
+def test_position():
+    vm = ViewMapper(viewangle=180)
+    res = 1000
+    img = vm.pixelrays(res)
+    fimg = img.reshape(-1, 3)
+    cos = vm.ctheta(fimg)
+    pc = cos.reshape(res, res)
+    io.array2hdr(pc, "position_cos.hdr")
+    posfinder = PositionIndex()
+    posidx = posfinder.positions(vm, fimg).reshape(res, res)
+    pg = 1/np.square(posidx).reshape(res, res)
+    io.array2hdr(pg, "position_guth.hdr")
+    posfinder = PositionIndex(guth=False)
+    posidx = posfinder.positions(vm, fimg).reshape(res, res)
+    pk = 1/posidx.reshape(res, res)
+    io.array2hdr(pk, "position_kim.hdr")
+    position_kim = io.hdr2array("position_kim.hdr")
+    position_guth = io.hdr2array("position_guth.hdr")
+    position_cos = io.hdr2array("position_cos.hdr")
+    assert np.allclose(position_kim, pk, atol=4.8, rtol=.1)
+    assert np.allclose(position_guth, pg, atol=.5, rtol=.1)
+    assert np.allclose(position_cos, pc, atol=.6, rtol=.1)
+
